/* config.h.in.  Generated from configure.ac for CMake.  */

#ifndef CONFIG_H
#define CONFIG_H

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "h5part@lists.psi.ch"

/* Define to the full name of this package. */
#define PACKAGE_NAME "h5hut"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "VTK "

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "VTK"

/* Define to the home page for this package. */
/* #undef PACKAGE_URL */

/* Define to the version of this package. */
#define PACKAGE_VERSION "2.0.0rc7"

/* Additional feature flags set by CMake options */
#define USE_LARGE_INDICES
/* #undef H5_USE_LUSTRE */
/* #undef WITH_PARALLEL_H5GRID */

#endif /* CONFIG_H */
