// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#ifndef vtk_h5hut_h
#define vtk_h5hut_h

#define VTK_H5HUT(header) <vtkh5hut/src/include/header>

#endif
