#ifndef vtkioss_mangle_h
#define vtkioss_mangle_h

// Mangle all global namespaces
#define Iocatalyst  vtkioss_Iocatalyst
#define Iocgns      vtkioss_Iocgns
#define Ioex        vtkioss_Ioex
#define Ioexnl      vtkioss_Ioexnl
#define Iogn        vtkioss_Iogn
#define Iogs        vtkioss_Iogs
#define Iohb        vtkioss_Iohb
#define Ionull      vtkioss_Ionull
#define Ioss        vtkioss_Ioss
#define Iotr        vtkioss_Iotr

#endif
